@@ -7,7 +7,7 @@ import warnings
 from abc import ABC, abstractmethod
 from collections import defaultdict
 from functools import partial
-from typing import Any, Dict, Optional, Sequence, Union
+from typing import Any, Dict, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import torch
@@ -88,9 +88,7 @@ class BenchmarkDataset(ABC):
             model_id (str):
                 The full Hugging Face Hub path to the pretrained transformer model. The
                 specific model version to use can be added after the suffix '@':
-                "model_id@v1.0.0". It can be a branch name, a tag name, or a commit id
-                (currently only supported for Hugging Face models, and it defaults to
-                'main' for latest).
+                "model_id@v1.0.0". It can be a branch name, a tag name, or a commit id.
 
         Returns:
             dict:
@@ -98,7 +96,8 @@ class BenchmarkDataset(ABC):
                 the first dictionary and the aggregated scores in the second.
 
         Raises:
-            RuntimeError: If the extracted framework is not recognized.
+            RuntimeError:
+                If the extracted framework is not recognized.
         """
         # Fetch the model config
         model_config = get_model_config(
@@ -137,7 +136,7 @@ class BenchmarkDataset(ABC):
                 pass
 
         # Set variable with number of iterations
-        num_iter = 10
+        num_iter = 10 if not self.benchmark_config.testing else 2
 
         if model_config.framework in {"pytorch", "jax"}:
             return self._benchmark_pytorch_jax(
@@ -214,9 +213,6 @@ class BenchmarkDataset(ABC):
                 "note that this is currently under development, however."
             )
 
-        # Get bootstrap sample indices
-        test_bidxs = rng.integers(0, len(test), size=(num_iter, len(test)))
-
         # Extract the model and tokenizer
         model = model_dict["model"]
         tokenizer = model_dict["tokenizer"]
@@ -235,6 +231,13 @@ class BenchmarkDataset(ABC):
         except ValueError:
             raise InvalidBenchmark("Preprocessing of the dataset could not be done.")
 
+        # If we are testing then truncate the test set
+        if self.benchmark_config.testing:
+            test = test.select(range(128))
+
+        # Get bootstrap sample indices
+        test_bidxs = rng.integers(0, len(test), size=(num_iter, len(test)))
+
         # Get bootstrapped datasets
         tests = [
             Dataset.from_dict(test[test_bidxs[idx]])
@@ -262,10 +265,10 @@ class BenchmarkDataset(ABC):
             eval_steps=30,
             logging_steps=30,
             save_steps=30,
-            max_steps=10_000,
+            max_steps=10_000 if not self.benchmark_config.testing else 2,
             report_to="none",
             save_total_limit=1,
-            per_device_train_batch_size=32,
+            per_device_train_batch_size=32 if not self.benchmark_config.testing else 1,
             per_device_eval_batch_size=32,
             learning_rate=2e-5,
             warmup_ratio=0.01,
@@ -274,6 +277,7 @@ class BenchmarkDataset(ABC):
             optim="adamw_torch",
             seed=4242,
             bf16=torch.cuda.is_available() or not torch.backends.mps.is_available(),
+            no_cuda=self.benchmark_config.testing,
         )
 
         # Manually set `disable_tqdm` to `False` if `progress_bar` is `True`
@@ -301,32 +305,14 @@ class BenchmarkDataset(ABC):
                 # Otherwise we encountered an error, so we have to deal with it and try
                 # again
                 else:
-                    # Rename `itr_scores` to `e` to make it more readable
-                    e = itr_scores
+                    bs, ga = self._handle_error(
+                        e=itr_scores, training_args=training_args
+                    )
 
-                    # We assume that all these CUDA errors are caused by
-                    # insufficient GPU memory
-                    # TODO: Handle MPS out of memory as well
-                    cuda_errs = ["CUDA out of memory", "CUDA error"]
-
-                    # If it is an unknown error, then simply report it
-                    if all([err not in str(e) for err in cuda_errs]):
-                        raise InvalidBenchmark(str(e))
-
-                    # If it is a CUDA memory error, then reduce batch size and up
-                    # gradient accumulation
-                    bs = training_args.per_device_train_batch_size
-                    ga = training_args.gradient_accumulation_steps
-                    if bs == 1:
-                        raise InvalidBenchmark(
-                            "CUDA out of memory, even with a batch size of 1!"
-                        )
                     training_args.per_device_train_batch_size = bs // 2
                     training_args.per_device_eval_batch_size = bs // 2
                     training_args.gradient_accumulation_steps = ga * 2
 
-                    print("New batch size:", bs // 2)
-
             if "train" in itr_scores:
                 scores["train"].append(itr_scores["train"])
             scores["test"].append(itr_scores["test"])
@@ -340,10 +326,50 @@ class BenchmarkDataset(ABC):
         )
 
         # Garbage collection, to avoid memory issues
-        clear_memory([model, model_dict])
+        try:
+            del model
+        except UnboundLocalError:
+            pass
+        try:
+            del model_dict
+        except UnboundLocalError:
+            pass
+        clear_memory()
 
         return all_scores
 
+    def _handle_error(
+        self, e: Exception, training_args: TrainingArguments
+    ) -> Tuple[int, int]:
+        """Handle an error that occurred during the benchmarking process.
+
+        Args:
+            e (Exception):
+                The exception that was raised.
+            training_args (TrainingArguments):
+                The training arguments that were used.
+
+        Returns:
+            pair of int:
+                The batch size and gradient accumulation steps to use.
+        """
+        # We assume that all these CUDA errors are caused by
+        # insufficient GPU memory
+        # TODO: Handle MPS out of memory as well
+        cuda_errs = ["CUDA out of memory", "CUDA error"]
+
+        # If it is an unknown error, then simply report it
+        if all([err not in str(e) for err in cuda_errs]):
+            raise InvalidBenchmark(str(e))
+
+        # If it is a CUDA memory error, then reduce batch size and up
+        # gradient accumulation
+        bs = training_args.per_device_train_batch_size
+        ga = training_args.gradient_accumulation_steps
+        if bs == 1:
+            raise InvalidBenchmark("CUDA out of memory, even with a batch size of 1!")
+        return bs // 2, ga * 2
+
     def _benchmark_pytorch_jax_single_iteration(
         self,
         idx: int,
@@ -467,7 +493,15 @@ class BenchmarkDataset(ABC):
             return scores
 
         except (RuntimeError, ValueError, IndexError) as e:
-            clear_memory([model, model_dict])
+            try:
+                del model
+            except UnboundLocalError:
+                pass
+            try:
+                del model_dict
+            except UnboundLocalError:
+                pass
+            clear_memory()
             return e
 
     def _benchmark_spacy(
@@ -565,7 +599,15 @@ class BenchmarkDataset(ABC):
         )
 
         # Garbage collection, to avoid memory issues
-        clear_memory([model, model_dict])
+        try:
+            del model
+        except UnboundLocalError:
+            pass
+        try:
+            del model_dict
+        except UnboundLocalError:
+            pass
+        clear_memory()
 
         return all_scores
 
@@ -684,6 +726,10 @@ class BenchmarkDataset(ABC):
         """Load a PyTorch model.
 
         Args:
+            model_config (ModelConfig):
+                The configuration of the model.
+            from_flax (bool):
+                Whether the model is a Flax model.
 
         Returns:
             dict:
@@ -810,6 +856,8 @@ class BenchmarkDataset(ABC):
         """Load a spaCy model.
 
         Args:
+            model_config (ModelConfig):
+                The configuration of the model.
 
         Returns:
             dict:
